/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.scdh;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinColumns;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

@Entity
@Table(name = "DH_DOCUMENTOS")
@Inheritance(strategy = InheritanceType.JOINED)
@DiscriminatorColumn(
	name = "TIPO", discriminatorType = DiscriminatorType.INTEGER)
public abstract class Documento
{
	private int _codigo;
	private Arquivo _arquivo;
	private TipoDocumento _tipo;
	private PeriodoProducao _periodoProducao;
	private Pais _paisProducao;
	private Estado _estadoProducao;
	private Cidade _cidadeProducao;
	private String _resumo;
	private Acesso _acesso;
	private String _motivoRestricaoAcesso;
	private List<AssuntoDocumento> _assuntos =
		new ArrayList<AssuntoDocumento>();
	private List<ProdutorDocumento> _produtores =
		new ArrayList<ProdutorDocumento>();
	
	protected Documento()
	{
	}

	@Id
	@Column(name = "COD_DOCUMENTO")
	public int getCodigo()
	{
		return _codigo;
	}

	public void setCodigo(int codigo)
	{
		_codigo = codigo;
	}

	@ManyToOne
	@JoinColumn(name = "COD_ARQUIVO")
	@NotNull
	public Arquivo getArquivo()
	{
		return _arquivo;
	}

	public void setArquivo(Arquivo arquivo)
	{
		_arquivo = arquivo;
	}

	@NotNull
	public TipoDocumento getTipo()
	{
		return _tipo;
	}

	public void setTipo(TipoDocumento tipo)
	{
		_tipo = tipo;
	}

	public PeriodoProducao getPeriodoProducao()
	{
		return _periodoProducao;
	}

	public void setPeriodoProducao(PeriodoProducao periodoProducao)
	{
		_periodoProducao = periodoProducao;
	}

	@ManyToOne
	@JoinColumn(name = "COD_PAIS_PRODUCAO")
	public Pais getPaisProducao()
	{
		return _paisProducao;
	}

	public void setPaisProducao(Pais paisProducao)
	{
		_paisProducao = paisProducao;
	}

	// TODO: Informar referencedColumnName se necessrio
	@ManyToOne
	@JoinColumns({
		@JoinColumn(name = "COD_PAIS_ESTADO_PRODUCAO",
			referencedColumnName = "COD_PAIS"),
		@JoinColumn(name = "COD_ESTADO_ESTADO_PRODUCAO",
			referencedColumnName = "COD_ESTADO")
	})
	public Estado getEstadoProducao()
	{
		return _estadoProducao;
	}

	public void setEstadoProducao(Estado estadoProducao)
	{
		_estadoProducao = estadoProducao;
	}

	// TODO: Informar referencedColumnName se necessrio
	@ManyToOne
	@JoinColumns({
		@JoinColumn(name = "COD_PAIS_CIDADE_PRODUCAO",
			referencedColumnName = "COD_PAIS"),
		@JoinColumn(name = "COD_ESTADO_CIDADE_PRODUCAO",
			referencedColumnName = "COD_ESTADO"),
		@JoinColumn(name = "COD_CIDADE_CIDADE_PRODUCAO",
			referencedColumnName = "COD_CIDADE")
	})
	public Cidade getCidadeProducao()
	{
		return _cidadeProducao;
	}

	public void setCidadeProducao(Cidade cidadeProducao)
	{
		_cidadeProducao = cidadeProducao;
	}

	@Lob
	@NotNull
	public String getResumo()
	{
		return _resumo;
	}

	public void setResumo(String resumo)
	{
		_resumo = resumo;
	}

	@NotNull
	public Acesso getAcesso()
	{
		return _acesso;
	}

	public void setAcesso(Acesso acesso)
	{
		_acesso = acesso;
	}

	@Lob
	@Column(name = "MOTIVO_RESTRICAO_ACESSO")
	public String getMotivoRestricaoAcesso()
	{
		return _motivoRestricaoAcesso;
	}

	public void setMotivoRestricaoAcesso(String motivoRestricaoAcesso)
	{
		_motivoRestricaoAcesso = motivoRestricaoAcesso;
	}

	@OneToMany(mappedBy = "documento", cascade = {CascadeType.ALL})
	public List<AssuntoDocumento> getAssuntos()
	{
		return _assuntos;
	}

	public void setAssuntos(List<AssuntoDocumento> assuntos)
	{
		_assuntos = assuntos;
	}

	@OneToMany(mappedBy = "documento", cascade = {CascadeType.ALL},
		orphanRemoval = true)
	public List<ProdutorDocumento> getProdutores()
	{
		return _produtores;
	}

	public void setProdutores(List<ProdutorDocumento> produtores)
	{
		_produtores = produtores;
	}
}
