/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.scdh.web;

import java.io.Serializable;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.faces.FacesException;
import javax.faces.event.ValueChangeEvent;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import br.urca.scdh.IGerenciadorCadastro;

public abstract class Cadastro<T, C extends Number> implements Serializable
{
	private static final long serialVersionUID = 4507343831290246569L;
	
	private T _entidade;
	private C _chave;
	private EstadoDados _estadoDados = EstadoDados.CONSULTA;
	private boolean _entidadeExistente;
	private IGerenciadorCadastro<T, C> _gerenciador;
	
	public Cadastro()
	{
		_entidade = getGerenciador().criar();
	}
	
	public T getEntidade()
	{
		return _entidade;
	}
	
	void setEntidade(T entidade)
	{
		_entidade = entidade;
	}

	public C getChave()
	{
		return _chave;
	}

	public void setChave(C chave)
	{
		_chave = chave;
	}

	public EstadoDados getEstadoDados()
	{
		return _estadoDados;
	}
	
	public void setEstadoDados(EstadoDados estadoDados)
	{
		_estadoDados = estadoDados;
	}

	public boolean isEntidadeExistente()
	{
		return _entidadeExistente;
	}

	public void setEntidadeExistente(boolean entidadeExistente)
	{
		_entidadeExistente = entidadeExistente;
	}
	
	abstract Class<? extends IGerenciadorCadastro<T, C>> getClasseGerenciador();

	public void novo()
	{
		_estadoDados = EstadoDados.INCLUSAO;
		_entidadeExistente = false;
		_chave = null;
		Utils.focalizarComponente("form:focoNovo");
	}
	
	public void estadoDadosAlterado(ValueChangeEvent e)
	{
		EstadoDados estadoDados = (EstadoDados) e.getNewValue();
		if (estadoDados == EstadoDados.INCLUSAO)
		{
			_chave = null;
		}
	}
	
	private IGerenciadorCadastro<T, C> getGerenciador()
	{
		if (_gerenciador == null)
		{
			_gerenciador = criarGerenciador();
		}
		return _gerenciador;
	}

	private IGerenciadorCadastro<T, C> criarGerenciador()
	{
		try
		{
			BeanManager gerenciadorBean =
				(BeanManager) new InitialContext().lookup(
				"java:comp/BeanManager");
			@SuppressWarnings("unchecked")
			Bean<IGerenciadorCadastro<T, C>> bean =
				(Bean<IGerenciadorCadastro<T, C>>) gerenciadorBean.getBeans(
				getClasseGerenciador()).iterator().next();
			CreationalContext<IGerenciadorCadastro<T, C>> contexto =
				gerenciadorBean.createCreationalContext(bean);
			@SuppressWarnings("unchecked")
			IGerenciadorCadastro<T, C> gerenciador =
				(IGerenciadorCadastro<T, C>) gerenciadorBean.getReference(
				bean, getClasseGerenciador(), contexto);
			return gerenciador;
		}
		catch (NamingException ex)
		{
			throw new FacesException(ex);
		}
	}
	
	public enum EstadoDados
	{
		CONSULTA,
		INCLUSAO,
		EDICAO
	}
}
